#! /usr/bin/env ruby
# coding: utf-8

require "helper"
#require "fileutils"
#require "test/unit"
#require "crysna/modelcell.rb"

$tolerance = 1.0E-10

class Crysna::ModelStructure
  public :to_coord
  #public :tetrahedra
  #public :octahedra
  public :polyhedra
  #public :periodically_include?
  public :choose_zyx
end

class TC_ModelStructureCell < Test::Unit::TestCase

  TMP_MODEL_YAML = "test/tmp_model.yaml"

  FRAME_ELEMENTS = [ 0 ]
  FRAME_SITES = {
    "FS_000" => [ 0.50, 0.50, 0.50],
    "FS_001" => [ 0.50, 0.50, 0.51],
    "FS_003" => [ 0.50, 0.50, 0.53],
    "FS_020" => [ 0.50, 0.52, 0.50],
    "FS_060" => [ 0.50, 0.56, 0.50],
    "FS_300" => [ 0.53, 0.50, 0.50],
    "FS_900" => [ 0.59, 0.50, 0.50],
  }
  SYMMETRY_OPERATIONS = [
    { "rotation" => [[ 1, 0, 0], [ 0, 1, 0], [ 0, 0, 1]],
      "translation" => [ 0.0, 0.0, 0.0]
    },
    { "rotation" => [[-1, 0, 0], [ 0,-1, 0], [ 0, 0,-1]],
      "translation" => [ 0.0, 0.0, 0.0]
    }
  ]
  TETRAHEDRAL_SITES = {
    "Te_000"=> [
      ["FS_000", [0, 0, 0]],
      ["FS_000", [0, 0, 1]],
      ["FS_000", [0, 1, 0]],
      ["FS_000", [1, 0, 0]]
    ],
    "Te_001"=> [
      ["FS_000", [ 0, 0, 0]],
      ["FS_000", [ 0, 0,-1]],
      ["FS_000", [ 0,-1, 0]],
      ["FS_000", [-1, 0, 0]]
    ]
  }
  OCTAHEDRAL_SITES = {
    "Oc_000"=> [
      [["FS_000", [-1,-1, 0]], ["FS_000", [ 0, 0,-1]]],
      [["FS_000", [-1, 0,-1]], ["FS_000", [ 0,-1, 0]]],
      [["FS_000", [ 0,-1,-1]], ["FS_000", [-1, 0, 0]]]
    ]
  }

  TE000_000 = Crysna::SiteWithPosition.new("Te_000", [ 0, 0, 0], [ 0.25, 0.25, 0.25])
  TE000_001 = Crysna::SiteWithPosition.new("Te_000", [ 0, 0, 1], [ 0.25, 0.25, 0.25])
  TE000_00M = Crysna::SiteWithPosition.new("Te_000", [ 0, 0,-1], [ 0.25, 0.25, 0.25])
  TE000_010 = Crysna::SiteWithPosition.new("Te_000", [ 0, 1, 0], [ 0.25, 0.25, 0.25])
  TE000_011 = Crysna::SiteWithPosition.new("Te_000", [ 0, 1, 1], [ 0.25, 0.25, 0.25])
  TE000_01M = Crysna::SiteWithPosition.new("Te_000", [ 0, 1,-1], [ 0.25, 0.25, 0.25])
  TE000_0M0 = Crysna::SiteWithPosition.new("Te_000", [ 0,-1, 0], [ 0.25, 0.25, 0.25])
  TE000_0M1 = Crysna::SiteWithPosition.new("Te_000", [ 0,-1, 1], [ 0.25, 0.25, 0.25])
  TE000_0MM = Crysna::SiteWithPosition.new("Te_000", [ 0,-1,-1], [ 0.25, 0.25, 0.25])
  TE000_100 = Crysna::SiteWithPosition.new("Te_000", [ 1, 0, 0], [ 0.25, 0.25, 0.25])
  TE000_101 = Crysna::SiteWithPosition.new("Te_000", [ 1, 0, 1], [ 0.25, 0.25, 0.25])
  TE000_10M = Crysna::SiteWithPosition.new("Te_000", [ 1, 0,-1], [ 0.25, 0.25, 0.25])
  TE000_110 = Crysna::SiteWithPosition.new("Te_000", [ 1, 1, 0], [ 0.25, 0.25, 0.25])
  TE000_111 = Crysna::SiteWithPosition.new("Te_000", [ 1, 1, 1], [ 0.25, 0.25, 0.25])
  TE000_11M = Crysna::SiteWithPosition.new("Te_000", [ 1, 1,-1], [ 0.25, 0.25, 0.25])
  TE000_1M0 = Crysna::SiteWithPosition.new("Te_000", [ 1,-1, 0], [ 0.25, 0.25, 0.25])
  TE000_1M1 = Crysna::SiteWithPosition.new("Te_000", [ 1,-1, 1], [ 0.25, 0.25, 0.25])
  TE000_1MM = Crysna::SiteWithPosition.new("Te_000", [ 1,-1,-1], [ 0.25, 0.25, 0.25])
  TE000_M00 = Crysna::SiteWithPosition.new("Te_000", [-1, 0, 0], [ 0.25, 0.25, 0.25])
  TE000_M01 = Crysna::SiteWithPosition.new("Te_000", [-1, 0, 1], [ 0.25, 0.25, 0.25])
  TE000_M0M = Crysna::SiteWithPosition.new("Te_000", [-1, 0,-1], [ 0.25, 0.25, 0.25])
  TE000_M10 = Crysna::SiteWithPosition.new("Te_000", [-1, 1, 0], [ 0.25, 0.25, 0.25])
  TE000_M11 = Crysna::SiteWithPosition.new("Te_000", [-1, 1, 1], [ 0.25, 0.25, 0.25])
  TE000_M1M = Crysna::SiteWithPosition.new("Te_000", [-1, 1,-1], [ 0.25, 0.25, 0.25])
  TE000_MM0 = Crysna::SiteWithPosition.new("Te_000", [-1,-1, 0], [ 0.25, 0.25, 0.25])
  TE000_MM1 = Crysna::SiteWithPosition.new("Te_000", [-1,-1, 1], [ 0.25, 0.25, 0.25])
  TE000_MMM = Crysna::SiteWithPosition.new("Te_000", [-1,-1,-1], [ 0.25, 0.25, 0.25])
                                                        
  TE001_000 = Crysna::SiteWithPosition.new("Te_001", [ 0, 0, 0], [-0.25,-0.25,-0.25])
  TE001_001 = Crysna::SiteWithPosition.new("Te_001", [ 0, 0, 1], [-0.25,-0.25,-0.25])
  TE001_00M = Crysna::SiteWithPosition.new("Te_001", [ 0, 0,-1], [-0.25,-0.25,-0.25])
  TE001_010 = Crysna::SiteWithPosition.new("Te_001", [ 0, 1, 0], [-0.25,-0.25,-0.25])
  TE001_011 = Crysna::SiteWithPosition.new("Te_001", [ 0, 1, 1], [-0.25,-0.25,-0.25])
  TE001_01M = Crysna::SiteWithPosition.new("Te_001", [ 0, 1,-1], [-0.25,-0.25,-0.25])
  TE001_0M0 = Crysna::SiteWithPosition.new("Te_001", [ 0,-1, 0], [-0.25,-0.25,-0.25])
  TE001_0M1 = Crysna::SiteWithPosition.new("Te_001", [ 0,-1, 1], [-0.25,-0.25,-0.25])
  TE001_0MM = Crysna::SiteWithPosition.new("Te_001", [ 0,-1,-1], [-0.25,-0.25,-0.25])
  TE001_100 = Crysna::SiteWithPosition.new("Te_001", [ 1, 0, 0], [-0.25,-0.25,-0.25])
  TE001_101 = Crysna::SiteWithPosition.new("Te_001", [ 1, 0, 1], [-0.25,-0.25,-0.25])
  TE001_10M = Crysna::SiteWithPosition.new("Te_001", [ 1, 0,-1], [-0.25,-0.25,-0.25])
  TE001_110 = Crysna::SiteWithPosition.new("Te_001", [ 1, 1, 0], [-0.25,-0.25,-0.25])
  TE001_111 = Crysna::SiteWithPosition.new("Te_001", [ 1, 1, 1], [-0.25,-0.25,-0.25])
  TE001_11M = Crysna::SiteWithPosition.new("Te_001", [ 1, 1,-1], [-0.25,-0.25,-0.25])
  TE001_1M0 = Crysna::SiteWithPosition.new("Te_001", [ 1,-1, 0], [-0.25,-0.25,-0.25])
  TE001_1M1 = Crysna::SiteWithPosition.new("Te_001", [ 1,-1, 1], [-0.25,-0.25,-0.25])
  TE001_1MM = Crysna::SiteWithPosition.new("Te_001", [ 1,-1,-1], [-0.25,-0.25,-0.25])
  TE001_M00 = Crysna::SiteWithPosition.new("Te_001", [-1, 0, 0], [-0.25,-0.25,-0.25])
  TE001_M01 = Crysna::SiteWithPosition.new("Te_001", [-1, 0, 1], [-0.25,-0.25,-0.25])
  TE001_M0M = Crysna::SiteWithPosition.new("Te_001", [-1, 0,-1], [-0.25,-0.25,-0.25])
  TE001_M10 = Crysna::SiteWithPosition.new("Te_001", [-1, 1, 0], [-0.25,-0.25,-0.25])
  TE001_M11 = Crysna::SiteWithPosition.new("Te_001", [-1, 1, 1], [-0.25,-0.25,-0.25])
  TE001_M1M = Crysna::SiteWithPosition.new("Te_001", [-1, 1,-1], [-0.25,-0.25,-0.25])
  TE001_MM0 = Crysna::SiteWithPosition.new("Te_001", [-1,-1, 0], [-0.25,-0.25,-0.25])
  TE001_MM1 = Crysna::SiteWithPosition.new("Te_001", [-1,-1, 1], [-0.25,-0.25,-0.25])
  TE001_MMM = Crysna::SiteWithPosition.new("Te_001", [-1,-1,-1], [-0.25,-0.25,-0.25])

  OC000_000 = Crysna::SiteWithPosition.new("Oc_000", [ 0, 0, 0], [-0.50,-0.50,-0.50])
  OC000_001 = Crysna::SiteWithPosition.new("Oc_000", [ 0, 0, 1], [-0.50,-0.50,-0.50])
  OC000_00M = Crysna::SiteWithPosition.new("Oc_000", [ 0, 0,-1], [-0.50,-0.50,-0.50])
  OC000_010 = Crysna::SiteWithPosition.new("Oc_000", [ 0, 1, 0], [-0.50,-0.50,-0.50])
  OC000_011 = Crysna::SiteWithPosition.new("Oc_000", [ 0, 1, 1], [-0.50,-0.50,-0.50])
  OC000_01M = Crysna::SiteWithPosition.new("Oc_000", [ 0, 1,-1], [-0.50,-0.50,-0.50])
  OC000_0M0 = Crysna::SiteWithPosition.new("Oc_000", [ 0,-1, 0], [-0.50,-0.50,-0.50])
  OC000_0M1 = Crysna::SiteWithPosition.new("Oc_000", [ 0,-1, 1], [-0.50,-0.50,-0.50])
  OC000_0MM = Crysna::SiteWithPosition.new("Oc_000", [ 0,-1,-1], [-0.50,-0.50,-0.50])
  OC000_100 = Crysna::SiteWithPosition.new("Oc_000", [ 1, 0, 0], [-0.50,-0.50,-0.50])
  OC000_101 = Crysna::SiteWithPosition.new("Oc_000", [ 1, 0, 1], [-0.50,-0.50,-0.50])
  OC000_10M = Crysna::SiteWithPosition.new("Oc_000", [ 1, 0,-1], [-0.50,-0.50,-0.50])
  OC000_110 = Crysna::SiteWithPosition.new("Oc_000", [ 1, 1, 0], [-0.50,-0.50,-0.50])
  OC000_111 = Crysna::SiteWithPosition.new("Oc_000", [ 1, 1, 1], [-0.50,-0.50,-0.50])
  OC000_11M = Crysna::SiteWithPosition.new("Oc_000", [ 1, 1,-1], [-0.50,-0.50,-0.50])
  OC000_1M0 = Crysna::SiteWithPosition.new("Oc_000", [ 1,-1, 0], [-0.50,-0.50,-0.50])
  OC000_1M1 = Crysna::SiteWithPosition.new("Oc_000", [ 1,-1, 1], [-0.50,-0.50,-0.50])
  OC000_1MM = Crysna::SiteWithPosition.new("Oc_000", [ 1,-1,-1], [-0.50,-0.50,-0.50])
  OC000_M00 = Crysna::SiteWithPosition.new("Oc_000", [-1, 0, 0], [-0.50,-0.50,-0.50])
  OC000_M01 = Crysna::SiteWithPosition.new("Oc_000", [-1, 0, 1], [-0.50,-0.50,-0.50])
  OC000_M0M = Crysna::SiteWithPosition.new("Oc_000", [-1, 0,-1], [-0.50,-0.50,-0.50])
  OC000_M10 = Crysna::SiteWithPosition.new("Oc_000", [-1, 1, 0], [-0.50,-0.50,-0.50])
  OC000_M11 = Crysna::SiteWithPosition.new("Oc_000", [-1, 1, 1], [-0.50,-0.50,-0.50])
  OC000_M1M = Crysna::SiteWithPosition.new("Oc_000", [-1, 1,-1], [-0.50,-0.50,-0.50])
  OC000_MM0 = Crysna::SiteWithPosition.new("Oc_000", [-1,-1, 0], [-0.50,-0.50,-0.50])
  OC000_MM1 = Crysna::SiteWithPosition.new("Oc_000", [-1,-1, 1], [-0.50,-0.50,-0.50])
  OC000_MMM = Crysna::SiteWithPosition.new("Oc_000", [-1,-1,-1], [-0.50,-0.50,-0.50])

  def setup
    hash = {
      "frame_elements" => FRAME_ELEMENTS,
      "frame_sites" => FRAME_SITES,
      "symmetry_operations" => SYMMETRY_OPERATIONS,
      "tetrahedral_sites" => TETRAHEDRAL_SITES,
      "octahedral_sites" => OCTAHEDRAL_SITES,
    }
    @mc00 = Crysna::ModelStructure.new(hash)
  end

  def test_load_file
    FileUtils.rm TMP_MODEL_YAML if File.exist? TMP_MODEL_YAML
    model = {
      "frame_elements"      => FRAME_ELEMENTS,
      "frame_sites"         => FRAME_SITES,
      "symmetry_operations" => SYMMETRY_OPERATIONS,
      "tetrahedral_sites"   => TETRAHEDRAL_SITES,
      "octahedral_sites"    => OCTAHEDRAL_SITES,
    }
    File.open(TMP_MODEL_YAML, "w") { |io| YAML.dump(model, io) }
    tmp = Crysna::ModelStructure.load_file(TMP_MODEL_YAML)
    #
    # 横着
    assert_equal(FRAME_ELEMENTS , tmp.frame_elements)
    assert_equal(7              , tmp.frame_sites.size)
    assert_equal(2              , tmp.symmetry_operations.size)
    #assert_equal(2             , tmp.tetrahedral_sites.size)
    #assert_equal(1             , tmp.octahedral_sites.size)
    assert_equal(3              , tmp.polyhedra.size)
    assert_equal(3              , tmp.site_names.size)
    #
    FileUtils.rm TMP_MODEL_YAML if File.exist? TMP_MODEL_YAML
  end

  def test_initialize
    assert_equal(Crysna::ModelStructure, @mc00.class)

    #未知の要素があればエラー。
    hash = {}
    hash["frame_elements"]          = FRAME_ELEMENTS
    hash["frame_sites"]                 = FRAME_SITES
    hash["symmetry_operations"] = SYMMETRY_OPERATIONS
    hash["tetrahedral_sites"]       = TETRAHEDRAL_SITES
    hash["octahedral_sites"]        = OCTAHEDRAL_SITES
    hash["something"]        = nil
    assert_raise(Crysna::ModelStructure::InitializeError){
      Crysna::ModelStructure.new( hash )
    }

    # 未知の要素を含む 5個の要素。当然エラー。
    hash = {}
    hash["frame_elements"]          = FRAME_ELEMENTS
    hash["frame_sites"]                 = FRAME_SITES
    hash["symmetry_operations"] = SYMMETRY_OPERATIONS
    hash["tetrahedral_sites"]       = TETRAHEDRAL_SITES
    hash["something"]        = nil
    assert_raise(Crysna::ModelStructure::InitializeError){
      Crysna::ModelStructure.new( hash )
    }

    ## 要素欠けは例外ではないと扱うことにした。[2014-12-09]
    #hash = {}
    #hash["frame_elements"]          = FRAME_ELEMENTS
    #hash["frame_sites"]                 = FRAME_SITES
    ##hash["symmetry_operations"] = SYMMETRY_OPERATIONS
    #hash["tetrahedral_sites"]       = TETRAHEDRAL_SITES
    #hash["octahedral_sites"]        = OCTAHEDRAL_SITES
    #assert_raise(Crysna::ModelStructure::InitializeError){
    #    Crysna::ModelStructure.new( hash )
    #}

    #hash = {}
    #hash["frame_elements"]          = FRAME_ELEMENTS
    ##hash["frame_sites"]                 = FRAME_SITES
    #hash["symmetry_operations"] = SYMMETRY_OPERATIONS
    #hash["tetrahedral_sites"]       = TETRAHEDRAL_SITES
    #hash["octahedral_sites"]        = OCTAHEDRAL_SITES
    #assert_raise(Crysna::ModelStructure::InitializeError){
    #    Crysna::ModelStructure.new( hash )
    #}

    #hash = {}
    ##hash["frame_elements"]          = FRAME_ELEMENTS
    #hash["frame_sites"]                 = FRAME_SITES
    #hash["symmetry_operations"] = SYMMETRY_OPERATIONS
    #hash["tetrahedral_sites"]       = TETRAHEDRAL_SITES
    #hash["octahedral_sites"]        = OCTAHEDRAL_SITES
    #assert_raise(Crysna::ModelStructure::InitializeError){
    #    Crysna::ModelStructure.new( hash )
    #}

    # OctahedralSites がなくても通過する。
    # チェックは体積のみなので、重なっていても検出できない。
    tmp = [
        ["FS_000", [0, 0, 0]], ["FS_000", [1, 0, 0]],
        ["FS_000", [0, 1, 0]], ["FS_000", [0, 0, 1]]
      ]
    tetrahedral_sites = {
      "Te_000"=> tmp,
      "Te_001"=> tmp,
      "Te_02"=> tmp,
      "Te_03"=> tmp,
      "Te_04"=> tmp,
      "Te_05"=> tmp,
    }
    hash = {}
    hash["frame_elements"]          = FRAME_ELEMENTS
    hash["frame_sites"]                 = FRAME_SITES
    hash["symmetry_operations"] = SYMMETRY_OPERATIONS
    hash["tetrahedral_sites"]       = tetrahedral_sites
    #hash["octahedral_sites"]        = OCTAHEDRAL_SITES
    assert_nothing_raised{ Crysna::ModelStructure.new( hash ) }

    # TetrahedralSites がなくても通過する。
    # チェックは体積のみなので、重なっていても検出できない。
    frame_sites = {
      "FS_000"=>[0.0, 0.0, 0.0],
      "FS_001"=>[0.5, 0.5, 0.0],
    }
    tmp = [
        [["FS_000", [0, 0, 0]], ["FS_000", [1, 1, 0]]],
        [["FS_000", [1, 0, 0]], ["FS_000", [0, 1, 0]]],
        [["FS_001", [0, 0, 0]], ["FS_000", [0, 0, 1]]],
      ]
    octahedral_sites = {
      "Oc_000" => tmp,
      "Oc_01" => tmp,
      "Oc_02" => tmp,
    }
    hash = {}
    hash["frame_elements"]          = FRAME_ELEMENTS
    hash["frame_sites"]                 = frame_sites
    hash["symmetry_operations"] = SYMMETRY_OPERATIONS
    #hash["tetrahedral_sites"]   = TETRAHEDRAL_SITES
    hash["octahedral_sites"]        = octahedral_sites
    Crysna::ModelStructure.new( hash )
    assert_nothing_raised{ Crysna::ModelStructure.new( hash ) }

    assert_equal(Mageo::Vector3DInternal, @mc00.frame_sites["FS_000"].class )

    #Check coodinates are 0 <= x,y,z < 1
    frame_elements = [ 0 ]
    frame_sites = { "FS_000" => [ 1.00, 0.00, 0.00], }
    symmetry_operations = [
      { "rotation" => [[ 1, 0, 0], [ 0, 1, 0], [ 0, 0, 1]],
        "translation" => [ 0.0, 0.0, 0.0]
      }
    ]
    tetrahedral_sites = {
      "Te_000"=> [
        ["FS_000", [0, 0, 0]],
        ["FS_000", [0, 0, 1]],
        ["FS_000", [0, 1, 0]],
        ["FS_000", [1, 0, 0]]
      ],
      "Te_001"=> [
        ["FS_000", [ 0, 0, 0]],
        ["FS_000", [ 0, 0,-1]],
        ["FS_000", [ 0,-1, 0]],
        ["FS_000", [-1, 0, 0]]
      ]
    }
    octahedral_sites = {
      "Oc_000"=> [
        [["FS_000", [-1,-1, 0]], ["FS_000", [ 0, 0,-1]]],
        [["FS_000", [-1, 0,-1]], ["FS_000", [ 0,-1, 0]]],
        [["FS_000", [ 0,-1,-1]], ["FS_000", [-1, 0, 0]]]
      ]
    }

    ## symmetry operation は外部に書くこともあるので例外から外した。
    ## raise error if invalid symmetry operation
    #symmetry_operations = [
    #    { "rotation" => [[ 1, 0, 0], [ 0, 1, 0], [ 0, 0, 1]],
    #        "translation" => [ 0.0, 0.0, 0.0]
    #    },
    #    { "rotation" => [[-1, 0, 0], [ 0,-1, 0], [ 0, 0,-1]],
    #        "translation" => [ 0.0, 0.0, 0.0]
    #    },
    #    { }
    #]
    #hash = {}
    #hash["frame_elements"]          = FRAME_ELEMENTS
    #hash["frame_sites"]                 = FRAME_SITES
    #hash["symmetry_operations"] = symmetry_operations
    #hash["tetrahedral_sites"]       = TETRAHEDRAL_SITES
    #hash["octahedral_sites"]        = OCTAHEDRAL_SITES
    ##pp hash
    #assert_raise(Crysna::ModelStructure::InitializeError){
    #    Crysna::ModelStructure.new( hash )
    #}

    #hash = {
    #    "frame_elements" => frame_elements,
    #    "frame_sites" => frame_sites,
    #    "symmetry_operations" => symmetry_operations,
    #    "tetrahedral_sites" => tetrahedral_sites,
    #    "octahedral_sites" => octahedral_sites,
    #}
    #assert_raise(Crysna::ModelStructure::SiteRangeError){ Crysna::ModelStructure.new( hash) }

    #frame_sites = {"FS_000" => [ 0.00, 1.00, 0.00] }
    #hash = {
    #    "frame_elements" => frame_elements,
    #    "frame_sites" => frame_sites,
    #    "symmetry_operations" => symmetry_operations,
    #    "tetrahedral_sites" => tetrahedral_sites,
    #    "octahedral_sites" => octahedral_sites,
    #}
    #assert_raise(Crysna::ModelStructure::SiteRangeError){ Crysna::ModelStructure.new( hash) }

    #frame_sites = { "FS_000" => [ 0.00, 0.00, 1.00], }
    #hash = {
    #    "frame_elements" => frame_elements,
    #    "frame_sites" => frame_sites,
    #    "symmetry_operations" => symmetry_operations,
    #    "tetrahedral_sites" => tetrahedral_sites,
    #    "octahedral_sites" => octahedral_sites,
    #}
    #assert_raise(Crysna::ModelStructure::SiteRangeError){ Crysna::ModelStructure.new( hash) }

    ##Octahedral is 
    #frame_sites = { "FS_000" => [ 0.50, 0.50, 0.50], }
    #octahedral_sites = {
    #    "Oc_000"=> [
    #        [["FS_000", [ 1, 1, 0]], ["FS_000", [ 0, 0, 1]]],
    #        [["FS_000", [ 1, 0, 1]], ["FS_000", [ 0, 1, 0]]],
    #        [["FS_000", [ 0, 1, 1]], ["FS_000", [ 1, 0, 0]]]
    #    ]
    #}
    #hash = {
    #    "frame_elements" => frame_elements,
    #    "frame_sites" => frame_sites,
    #    "symmetry_operations" => symmetry_operations,
    #    "tetrahedral_sites" => tetrahedral_sites,
    #    "octahedral_sites" => octahedral_sites,
    #}
    #assert_raise(Crysna::ModelStructure::SiteRangeError){ Crysna::ModelStructure.new( hash) }

  end

  def test_symmetry_operations
    assert_equal(
      [
        { "rotation" => [[ 1, 0, 0], [ 0, 1, 0], [ 0, 0, 1]],
          "translation" => [ 0.0, 0.0, 0.0]
        },
        { "rotation" => [[-1, 0, 0], [ 0,-1, 0], [ 0, 0,-1]],
          "translation" => [ 0.0, 0.0, 0.0]
        }
      ],
      @mc00.symmetry_operations
    )
  end

  def test_frame_elements
    assert_equal([0], @mc00.frame_elements)
  end

  def test_frame_sites
    tmp = @mc00.frame_sites
    assert_equal("FS_000", tmp.keys[0])
    assert_equal("FS_001", tmp.keys[1])
    assert_equal("FS_003", tmp.keys[2])
    assert_equal("FS_020", tmp.keys[3])
    assert_equal("FS_060", tmp.keys[4])
    assert_equal("FS_300", tmp.keys[5])
    assert_equal("FS_900", tmp.keys[6])
    assert_equal(Mageo::Vector3DInternal, tmp["FS_000"].class)
    assert_equal(Mageo::Vector3DInternal, tmp["FS_001"].class)
    assert_equal(Mageo::Vector3DInternal, tmp["FS_003"].class)
    assert_equal(Mageo::Vector3DInternal, tmp["FS_020"].class)
    assert_equal(Mageo::Vector3DInternal, tmp["FS_060"].class)
    assert_equal(Mageo::Vector3DInternal, tmp["FS_300"].class)
    assert_equal(Mageo::Vector3DInternal, tmp["FS_900"].class)
    #pp 0.10
    #pp tmp["FS_000"][0]
    #pp $tolerance
    assert_in_delta(0.50, tmp["FS_000"][0], $tolerance)
    assert_in_delta(0.50, tmp["FS_000"][1], $tolerance)
    assert_in_delta(0.50, tmp["FS_000"][2], $tolerance)
    assert_in_delta(0.50, tmp["FS_001"][0], $tolerance)
    assert_in_delta(0.50, tmp["FS_001"][1], $tolerance)
    assert_in_delta(0.51, tmp["FS_001"][2], $tolerance)
    assert_in_delta(0.50, tmp["FS_003"][0], $tolerance)
    assert_in_delta(0.50, tmp["FS_003"][1], $tolerance)
    assert_in_delta(0.53, tmp["FS_003"][2], $tolerance)
    assert_in_delta(0.50, tmp["FS_020"][0], $tolerance)
    assert_in_delta(0.52, tmp["FS_020"][1], $tolerance)
    assert_in_delta(0.50, tmp["FS_020"][2], $tolerance)
    assert_in_delta(0.50, tmp["FS_060"][0], $tolerance)
    assert_in_delta(0.56, tmp["FS_060"][1], $tolerance)
    assert_in_delta(0.50, tmp["FS_060"][2], $tolerance)
    assert_in_delta(0.53, tmp["FS_300"][0], $tolerance)
    assert_in_delta(0.50, tmp["FS_300"][1], $tolerance)
    assert_in_delta(0.50, tmp["FS_300"][2], $tolerance)
    assert_in_delta(0.59, tmp["FS_900"][0], $tolerance)
    assert_in_delta(0.50, tmp["FS_900"][1], $tolerance)
    assert_in_delta(0.50, tmp["FS_900"][2], $tolerance)
    assert_equal(7, tmp.size)
  end

  def test_ideal_interstitial_sites
    results = @mc00.ideal_interstitial_sites
    assert_equal(Mageo::Vector3DInternal, results["Te_000"].class)
    assert_in_delta( 0.75 , results["Te_000"][0], $tolerance)
    assert_in_delta( 0.75 , results["Te_000"][1], $tolerance)
    assert_in_delta( 0.75 , results["Te_000"][2], $tolerance)
    assert_equal(Mageo::Vector3DInternal, results["Te_001"].class)
    assert_in_delta( 0.25 , results["Te_001"][0], $tolerance)
    assert_in_delta( 0.25 , results["Te_001"][1], $tolerance)
    assert_in_delta( 0.25 , results["Te_001"][2], $tolerance)
    assert_equal(Mageo::Vector3DInternal, results["Oc_000"].class)
    assert_in_delta( 0.00 , results["Oc_000"][0], $tolerance)
    assert_in_delta( 0.00 , results["Oc_000"][1], $tolerance)
    assert_in_delta( 0.00 , results["Oc_000"][2], $tolerance)
  end

  def test_polyhedra
    results = @mc00.polyhedra
    assert_equal(["Te_000", "Te_001", "Oc_000"], results.keys)
    assert_equal(Mageo::Tetrahedron, results["Te_000"].class)
    assert_equal(Mageo::Vector3D, results["Te_000"].vertices[0].class)
    assert_in_delta( 0.50, results["Te_000"].vertices[0][0], $tolerance)
    assert_in_delta( 0.50, results["Te_000"].vertices[0][1], $tolerance)
    assert_in_delta( 0.50, results["Te_000"].vertices[0][2], $tolerance)
    assert_in_delta( 0.50, results["Te_000"].vertices[1][0], $tolerance)
    assert_in_delta( 0.50, results["Te_000"].vertices[1][1], $tolerance)
    assert_in_delta( 1.50, results["Te_000"].vertices[1][2], $tolerance)
    assert_in_delta( 0.50, results["Te_000"].vertices[2][0], $tolerance)
    assert_in_delta( 1.50, results["Te_000"].vertices[2][1], $tolerance)
    assert_in_delta( 0.50, results["Te_000"].vertices[2][2], $tolerance)
    assert_in_delta( 1.50, results["Te_000"].vertices[3][0], $tolerance)
    assert_in_delta( 0.50, results["Te_000"].vertices[3][1], $tolerance)
    assert_in_delta( 0.50, results["Te_000"].vertices[3][2], $tolerance)
    #
    assert_equal(Mageo::Tetrahedron, results["Te_001"].class)
    assert_equal(Mageo::Vector3D, results["Te_001"].vertices[0].class)
    assert_in_delta( 0.50, results["Te_001"].vertices[0][0], $tolerance)
    assert_in_delta( 0.50, results["Te_001"].vertices[0][1], $tolerance)
    assert_in_delta( 0.50, results["Te_001"].vertices[0][2], $tolerance)
    assert_in_delta( 0.50, results["Te_001"].vertices[1][0], $tolerance)
    assert_in_delta( 0.50, results["Te_001"].vertices[1][1], $tolerance)
    assert_in_delta(-0.50, results["Te_001"].vertices[1][2], $tolerance)
    assert_in_delta( 0.50, results["Te_001"].vertices[2][0], $tolerance)
    assert_in_delta(-0.50, results["Te_001"].vertices[2][1], $tolerance)
    assert_in_delta( 0.50, results["Te_001"].vertices[2][2], $tolerance)
    assert_in_delta(-0.50, results["Te_001"].vertices[3][0], $tolerance)
    assert_in_delta( 0.50, results["Te_001"].vertices[3][1], $tolerance)
    assert_in_delta( 0.50, results["Te_001"].vertices[3][2], $tolerance)
    #
    assert_equal(Mageo::Octahedron, results["Oc_000"].class)
    assert_equal(Mageo::Vector3D, results["Oc_000"].vertices[0].class)
    assert_in_delta(-0.50, results["Oc_000"].vertices[0][0], $tolerance)
    assert_in_delta(-0.50, results["Oc_000"].vertices[0][1], $tolerance)
    assert_in_delta( 0.50, results["Oc_000"].vertices[0][2], $tolerance)
    assert_in_delta( 0.50, results["Oc_000"].vertices[1][0], $tolerance)
    assert_in_delta( 0.50, results["Oc_000"].vertices[1][1], $tolerance)
    assert_in_delta(-0.50, results["Oc_000"].vertices[1][2], $tolerance)
    assert_in_delta(-0.50, results["Oc_000"].vertices[2][0], $tolerance)
    assert_in_delta( 0.50, results["Oc_000"].vertices[2][1], $tolerance)
    assert_in_delta(-0.50, results["Oc_000"].vertices[2][2], $tolerance)
    assert_in_delta( 0.50, results["Oc_000"].vertices[3][0], $tolerance)
    assert_in_delta(-0.50, results["Oc_000"].vertices[3][1], $tolerance)
    assert_in_delta( 0.50, results["Oc_000"].vertices[3][2], $tolerance)
    assert_in_delta( 0.50, results["Oc_000"].vertices[4][0], $tolerance)
    assert_in_delta(-0.50, results["Oc_000"].vertices[4][1], $tolerance)
    assert_in_delta(-0.50, results["Oc_000"].vertices[4][2], $tolerance)
    assert_in_delta(-0.50, results["Oc_000"].vertices[5][0], $tolerance)
    assert_in_delta( 0.50, results["Oc_000"].vertices[5][1], $tolerance)
    assert_in_delta( 0.50, results["Oc_000"].vertices[5][2], $tolerance)
  end

  def test_site_names
    assert_equal(["Te_000", "Te_001", "Oc_000"], @mc00.site_names)
  end

  def test_volume_check
    #サイト定義の四面体のいずれかが体積を持たなければエラー
    tetrahedral_sites = {
      "Te_000"=> [
        ["FS_000", [0, 0, 1]], ["FS_000", [1, 0, 1]],
        ["FS_000", [0, 1, 1]], ["FS_000", [0, 0, 1]]
      ],
      "Te_001"=> [
        ["FS_000", [1, 1, 1]], ["FS_000", [1, 0, 0]],
        ["FS_000", [0, 1, 0]], ["FS_000", [1, 1, 0]]
      ]
    }
    frame_sites = {
      "FS_000"=>[0.0, 0.0, 0.0], "FS_001"=>[0.5, 0.5, 0.0],
      "FS_002"=>[0.5, 0.0, 0.5], "FS_003"=>[0.0, 0.5, 0.5],
    }
    hash = {}
    hash["frame_elements"]          = FRAME_ELEMENTS
    hash["frame_sites"]                 = FRAME_SITES
    hash["symmetry_operations"] = SYMMETRY_OPERATIONS
    hash["tetrahedral_sites"]       = tetrahedral_sites
    hash["octahedral_sites"]        = OCTAHEDRAL_SITES
    model = Crysna::ModelStructure.new(hash)
    assert_raise(Crysna::ModelStructure::SiteVolumeError){model.check_volume}

    #サイト定義の八面体のいずれかが体積を持たなければエラー
    octahedral_sites = {
      "Oc_000"=> [
        [["FS_000", [0, 0, 0]], ["FS_000", [1, 1, 0]]],
        [["FS_000", [1, 0, 0]], ["FS_000", [0, 1, 0]]],
        [["FS_000", [1, 0, 0]], ["FS_000", [0, 1, 0]]]
      ]
    }
    frame_sites = {
      "FS_000"=>[0.0, 0.0, 0.0], "FS_001"=>[0.5, 0.5, 0.0],
      "FS_002"=>[0.5, 0.0, 0.5], "FS_003"=>[0.0, 0.5, 0.5],
    }
    hash = {}
    hash["frame_elements"]          = FRAME_ELEMENTS
    hash["frame_sites"]                 = FRAME_SITES
    hash["symmetry_operations"] = SYMMETRY_OPERATIONS
    hash["tetrahedral_sites"]       = TETRAHEDRAL_SITES
    hash["octahedral_sites"]        = octahedral_sites
    model = Crysna::ModelStructure.new(hash)
    assert_raise(Crysna::ModelStructure::SiteVolumeError){model.check_volume}

    #サイト定義の体積の総和がセル体積と一致しなければエラー
    tetrahedral_sites = {
      "Te_000"=> [
        ["FS_000", [0, 0, 0]], ["FS_000", [1, 0, 1]],
        ["FS_000", [0, 1, 1]], ["FS_000", [0, 0, 1]]
      ],
      "Te_001"=> [
        ["FS_000", [1, 1, 1]], ["FS_000", [1, 0, 0]],
        ["FS_000", [0, 1, 0]], ["FS_000", [1, 1, 0]]
      ],
      "Te_002"=> [
        ["FS_000", [1, 1, 1]], ["FS_000", [1, 0, 0]],
        ["FS_000", [0, 1, 0]], ["FS_000", [1, 1, 0]]
      ]
    }
    octahedral_sites = {
      "Oc_000"=> [
        [["FS_000", [0, 0, 0]], ["FS_000", [1, 1, 1]]],
        [["FS_000", [1, 0, 0]], ["FS_000", [0, 1, 1]]],
        [["FS_000", [1, 0, 1]], ["FS_000", [0, 1, 0]]]
      ]
    }
    hash = {}
    hash["frame_elements"]          = FRAME_ELEMENTS
    hash["frame_sites"]                 = FRAME_SITES
    hash["symmetry_operations"] = SYMMETRY_OPERATIONS
    hash["tetrahedral_sites"]       = tetrahedral_sites
    hash["octahedral_sites"]        = octahedral_sites
    model = Crysna::ModelStructure.new(hash)
    assert_raise(Crysna::ModelStructure::SiteVolumeError){model.check_volume}
  end

  def test_to_coord
    #assert_equal(Mageo::Vector3DInternal[0.50, -0.50, 2.50], @mc00.to_coord("FS_000", [0,-1,2]))
    results = @mc00.to_coord("FS_000", [0,-1,2])
    assert_equal(Mageo::Vector3D, results.class)
    assert_in_delta( 0.50, results[0], $tolerance)
    assert_in_delta(-0.50, results[1], $tolerance)
    assert_in_delta( 2.50, results[2], $tolerance)

    assert_raise(Crysna::ModelStructure::NoEntrySiteError) { @mc00.to_coord("SITE_A", [0,-1,2])}
  end

  def test_select_interstitial_sites27
    #In polyhedron
    assert_equal([TE000_000], @mc00.select_interstitial_sites27([ 0.51,  0.51,  0.51], $tolerance, StringIO.new))
    assert_equal([TE000_M00], @mc00.select_interstitial_sites27([ 0.40,  0.51,  0.51], $tolerance, StringIO.new))
    assert_equal([TE000_0M0], @mc00.select_interstitial_sites27([ 0.51,  0.40,  0.51], $tolerance, StringIO.new))
    assert_equal([TE000_00M], @mc00.select_interstitial_sites27([ 0.51,  0.51,  0.40], $tolerance, StringIO.new))
    #
    assert_equal([TE001_000], @mc00.select_interstitial_sites27([ 0.49,  0.49,  0.49], $tolerance, StringIO.new))
    assert_equal([TE001_100], @mc00.select_interstitial_sites27([ 0.60,  0.49,  0.49], $tolerance, StringIO.new))
    assert_equal([TE001_010], @mc00.select_interstitial_sites27([ 0.49,  0.60,  0.49], $tolerance, StringIO.new))
    assert_equal([TE001_001], @mc00.select_interstitial_sites27([ 0.49,  0.49,  0.60], $tolerance, StringIO.new))
    #
    assert_equal([OC000_011], @mc00.select_interstitial_sites27([ 0.49,  0.60,  0.60], $tolerance, StringIO.new))
    assert_equal([OC000_101], @mc00.select_interstitial_sites27([ 0.60,  0.49,  0.60], $tolerance, StringIO.new))
    assert_equal([OC000_110], @mc00.select_interstitial_sites27([ 0.60,  0.60,  0.49], $tolerance, StringIO.new))
    assert_equal([OC000_100], @mc00.select_interstitial_sites27([ 0.51,  0.40,  0.40], $tolerance, StringIO.new))
    assert_equal([OC000_010], @mc00.select_interstitial_sites27([ 0.40,  0.51,  0.40], $tolerance, StringIO.new))
    assert_equal([OC000_001], @mc00.select_interstitial_sites27([ 0.40,  0.40,  0.51], $tolerance, StringIO.new))

    #at vertex
    io = StringIO.new
    #io = $stdout
    results = @mc00.select_interstitial_sites27([ 0.50,  0.50,  0.50], $tolerance, io)
    assert_equal(14, results.size)
    assert_equal(TE000_00M, results[ 0]) #0,0,1
    assert_equal(TE000_0M0, results[ 1]) #1,0,0
    assert_equal(TE000_M00, results[ 2]) #0,1,0
    assert_equal(TE000_000, results[ 3]) #0,0,0
    assert_equal(TE001_000, results[ 4]) #1,1,1
    assert_equal(TE001_100, results[ 5]) #1,0,1
    assert_equal(TE001_010, results[ 6]) #0,1,1
    assert_equal(TE001_001, results[ 7]) #1,1,0
    assert_equal(OC000_100, results[ 8]) #1,0,1
    assert_equal(OC000_010, results[ 9]) #0,1,1
    assert_equal(OC000_110, results[10]) #0,0,1
    assert_equal(OC000_001, results[11]) #1,1,0
    assert_equal(OC000_101, results[12]) #1,0,0
    assert_equal(OC000_011, results[13]) #0,1,0

    #at edge
    io = StringIO.new
    #io = $stdout
    results = @mc00.select_interstitial_sites27([ 0.50,  0.00,  0.00], $tolerance, io)
    #pp results
    assert_equal(4, results.size)
    assert_equal(TE000_0MM, results[0])
    assert_equal(TE001_000, results[1])
    assert_equal(OC000_000, results[2])
    assert_equal(OC000_100, results[3])

    #on face
    results = @mc00.select_interstitial_sites27([ 0.75,  0.75,  0.00], $tolerance, StringIO.new)
    assert_equal( TE000_00M, results[0])
    assert_equal( OC000_110, results[1])
  end

  def test_find_site
    sites = {
      "FS_000" => [ 0.50, 0.50, 0.50].to_v3di,
      "FS_001" => [ 0.50, 0.50, 0.51].to_v3di,
      "FS_003" => [ 0.50, 0.50, 0.53].to_v3di,
      "FS_020" => [ 0.50, 0.52, 0.50].to_v3di,
      "FS_060" => [ 0.50, 0.56, 0.50].to_v3di,
      "FS_300" => [ 0.53, 0.50, 0.50].to_v3di,
      "FS_900" => [ 0.59, 0.50, 0.50].to_v3di,
    }

    #sites.each do |name, pos|
    #    @scatter_sites[name] = pos.to_v3di
    #end

    assert_equal(
      ["FS_000", [0,0,0]],
      @mc00.find_site([0.50, 0.50, 0.50], sites)
    )

    assert_equal(
      ["FS_900", [0,0,0]],
      @mc00.find_site([0.60, 0.50, 0.50], sites)
    )

    assert_equal(
      ["FS_900", [-1,0, 0]],
      @mc00.find_site([0.01, 0.01, 0.01], sites)
    )
  end

  #Return the nearest frame site.
  def test_find_nearest_frame_site
    assert_equal(
      ["FS_000", [0,0,0]],
      @mc00.find_nearest_frame_site([0.50,    0.50,    0.50])
    )

    assert_equal(
      ["FS_900", [0,0,0]],
      @mc00.find_nearest_frame_site([0.60,    0.50,    0.50])
    )

    assert_equal(
      ["FS_900", [-1,0, 0]],
      @mc00.find_nearest_frame_site([0.01,    0.01,    0.01])
    )
  end

  def test_find_interstitial_site
    #In polyhedron
    assert_equal("Te_000", @mc00.find_interstitial_site([ 0.51,  0.51,  0.51], $tolerance, StringIO.new))
    assert_equal("Te_000", @mc00.find_interstitial_site([ 0.40,  0.51,  0.51], $tolerance, StringIO.new))
    assert_equal("Te_000", @mc00.find_interstitial_site([ 0.51,  0.40,  0.51], $tolerance, StringIO.new))
    assert_equal("Te_000", @mc00.find_interstitial_site([ 0.51,  0.51,  0.40], $tolerance, StringIO.new))
    #
    assert_equal("Te_001", @mc00.find_interstitial_site([ 0.49,  0.49,  0.49], $tolerance, StringIO.new))
    assert_equal("Te_001", @mc00.find_interstitial_site([ 0.60,  0.49,  0.49], $tolerance, StringIO.new))
    assert_equal("Te_001", @mc00.find_interstitial_site([ 0.49,  0.60,  0.49], $tolerance, StringIO.new))
    assert_equal("Te_001", @mc00.find_interstitial_site([ 0.49,  0.49,  0.60], $tolerance, StringIO.new))
    #
    assert_equal("Oc_000", @mc00.find_interstitial_site([ 0.49,  0.60,  0.60], $tolerance, StringIO.new))
    assert_equal("Oc_000", @mc00.find_interstitial_site([ 0.60,  0.49,  0.60], $tolerance, StringIO.new))
    assert_equal("Oc_000", @mc00.find_interstitial_site([ 0.60,  0.60,  0.49], $tolerance, StringIO.new))
    assert_equal("Oc_000", @mc00.find_interstitial_site([ 0.51,  0.40,  0.40], $tolerance, StringIO.new))
    assert_equal("Oc_000", @mc00.find_interstitial_site([ 0.40,  0.51,  0.40], $tolerance, StringIO.new))
    assert_equal("Oc_000", @mc00.find_interstitial_site([ 0.40,  0.40,  0.51], $tolerance, StringIO.new))

    #at vertex
    assert_equal("Te_001", @mc00.find_interstitial_site([ 0.50,  0.50,  0.50], $tolerance, StringIO.new))

    #at edge
    assert_equal("Te_001", @mc00.find_interstitial_site([ 0.50,  0.00,  0.00], $tolerance, StringIO.new))

    #on face
    assert_equal("Oc_000", @mc00.find_interstitial_site([ 0.75,  0.75,  0.00], $tolerance, StringIO.new))
  end

  def test_choose_zyx
    candidates = [
      TE000_000, TE000_001, TE000_00M,
      TE000_010, TE000_011, TE000_01M,
      TE000_0M0, TE000_0M1, TE000_0MM,
      TE000_100, TE000_101, TE000_10M,
      TE000_110, TE000_111, TE000_11M,
      TE000_1M0, TE000_1M1, TE000_1MM,
      TE000_M00, TE000_M01, TE000_M0M,
      TE000_M10, TE000_M11, TE000_M1M,
      TE000_MM0, TE000_MM1, TE000_MMM,
    ]
    result = @mc00.choose_zyx(candidates)
    #pp result
    assert_equal(TE000_111, result)

    candidates = [
      TE001_000, TE001_001, TE001_00M,
      TE001_010, TE001_011, TE001_01M,
      TE001_0M0, TE001_0M1, TE001_0MM,
      TE001_100, TE001_101, TE001_10M,
      TE001_110, TE001_111, TE001_11M,
      TE001_1M0, TE001_1M1, TE001_1MM,
      TE001_M00, TE001_M01, TE001_M0M,
      TE001_M10, TE001_M11, TE001_M1M,
      TE001_MM0, TE001_MM1, TE001_MMM,
    ]
    result = @mc00.choose_zyx(candidates)
    assert_equal(TE001_111, result)

    candidates = [
      OC000_000, OC000_001, OC000_00M,
      OC000_010, OC000_011, OC000_01M,
      OC000_0M0, OC000_0M1, OC000_0MM,
      OC000_100, OC000_101, OC000_10M,
      OC000_110, OC000_111, OC000_11M,
      OC000_1M0, OC000_1M1, OC000_1MM,
      OC000_M00, OC000_M01, OC000_M0M,
      OC000_M10, OC000_M11, OC000_M1M,
      OC000_MM0, OC000_MM1, OC000_MMM,
    ]
    result = @mc00.choose_zyx(candidates)
    assert_equal(OC000_111, result)

    candidates = [
      TE000_000, TE001_000, OC000_000,
    ]
    result = @mc00.choose_zyx(candidates)
    assert_equal(TE000_000, result)

    candidates = [
      TE000_100, TE000_010, TE000_001, 
    ]
    result = @mc00.choose_zyx(candidates)
    assert_equal(TE000_001, result)
  end

end

