#! /usr/bin/env ruby
# coding: utf-8

require "helper"
#require "test/unit"
#require "pkg/klass.rb"

class TC_OptionManager < Test::Unit::TestCase
  def setup
    @o00 = Crysna::OptionManager.new
  end

  def test_parse_all
    ary = []
    @o00.parser.parse!(ary)
    corrects = {
      :cells=>"cells.yaml",
      :fitted=>"fittedstructure.yaml",
      :occupied=>"occupiedsite.yaml",
      :model=>"model.yaml",
      :symid=>"symidsite.yaml",
      :migration=>"sitemigration.yaml",
      :operation=>"siteoperation.yaml",
      :symmetry=>"symoperation.yaml",
      :unique  =>"siteunique.yaml",
      :single=>"sitesingle.yaml",
      :combination=> 'sitecombination.yaml',
      :candidate=> 'sitecandidate.yaml',
      :quiet=>false,
    }
    assert_equal(corrects, @o00.options)

    setup
    ary = %w(-m test.yaml)
    @o00.parser.parse!(ary)
    corrects = {
      :cells=>"cells.yaml",
      :fitted=>"fittedstructure.yaml",
      :occupied=>"occupiedsite.yaml",
      :model=>"test.yaml",
      :symid=>"symidsite.yaml",
      :migration=>"sitemigration.yaml",
      :operation=>"siteoperation.yaml",
      :symmetry=>"symoperation.yaml",
      :unique  =>"siteunique.yaml",
      :single=>"sitesingle.yaml",
      :combination=> 'sitecombination.yaml',
      :candidate=> 'sitecandidate.yaml',
      :quiet=>false,
    }
    assert_equal(corrects, @o00.options)

    setup
    ary = %w(-f test.yaml)
    @o00.parser.parse!(ary)
    assert_equal('test.yaml', @o00.options[:fitted])

    setup
    ary = %w(-M test.yaml)
    @o00.parser.parse!(ary)
    assert_equal('test.yaml', @o00.options[:migration])

    setup
    ary = %w(-o test.yaml)
    @o00.parser.parse!(ary)
    assert_equal('test.yaml', @o00.options[:operation])

    setup
    ary = %w(-c test.yaml)
    @o00.parser.parse!(ary)
    assert_equal('test.yaml', @o00.options[:cells])

    setup
    ary = %w(-i test.yaml)
    @o00.parser.parse!(ary)
    assert_equal('test.yaml', @o00.options[:symid])

    setup
    ary = %w(-y test.yaml)
    @o00.parser.parse!(ary)
    assert_equal('test.yaml', @o00.options[:symmetry])

    #### --model=-
    setup
    ary = %w(--model=-)
    @o00.parser.parse!(ary)
    assert_equal('-', @o00.options[:model])

    setup
    ary = %w(-m -)
    @o00.parser.parse!(ary)
    assert_equal('-', @o00.options[:model])

    setup
    ary = %w(-b test.yaml)
    @o00.parser.parse!(ary)
    assert_equal('test.yaml', @o00.options[:combination])

    setup
    ary = %w(-d test.yaml)
    @o00.parser.parse!(ary)
    assert_equal('test.yaml', @o00.options[:candidate])

    setup
    ary = %w(-s test.yaml)
    @o00.parser.parse!(ary)
    assert_equal('test.yaml', @o00.options[:single])

    setup
    ary = %w(-u test.yaml)
    @o00.parser.parse!(ary)
    assert_equal('test.yaml', @o00.options[:unique])

  end

  def test_init
    ary = []
    @o00.parser.parse!(ary)
    corrects = {
      :cells=>"cells.yaml",
      :fitted=>"fittedstructure.yaml",
      :occupied=>"occupiedsite.yaml",
      :model=>"model.yaml",
      :symid=>"symidsite.yaml",
      :migration=>"sitemigration.yaml",
      :operation=>"siteoperation.yaml",
      :symmetry=>"symoperation.yaml",
      :unique  =>"siteunique.yaml",
      :single=>"sitesingle.yaml",
      :combination=> 'sitecombination.yaml',
      :candidate=> 'sitecandidate.yaml',
      :quiet=>false,
    }
    assert_equal(corrects, @o00.options)

    o10 = Crysna::OptionManager.new([:model])
    ary = []
    o10.parser.parse!(ary)
    corrects = {
      :model=>"model.yaml",
    }
    assert_equal(corrects, o10.options)

    assert_raise(Crysna::OptionManager::INVARID_OPTION){
      Crysna::OptionManager.new([:cells, :unknown])
    }
  end

  def test_parse
    setup
    ary = %w(-m test.yaml abc)
    @o00.parse!(ary)
    corrects = {
      :cells=>"cells.yaml",
      :fitted=>"fittedstructure.yaml",
      :occupied=>"occupiedsite.yaml",
      :model=>"test.yaml",
      :symid=>"symidsite.yaml",
      :migration=>"sitemigration.yaml",
      :operation=>"siteoperation.yaml",
      :symmetry=>"symoperation.yaml",
      :unique  =>"siteunique.yaml",
      :single=>"sitesingle.yaml",
      :combination=> 'sitecombination.yaml',
      :candidate=> 'sitecandidate.yaml',
      :quiet=>false,
    }
    assert_equal(corrects, @o00.options)
    assert_equal(['abc'], ary)
  end
end

