#! /usr/bin/env ruby
# coding: utf-8

require "helper"
#require "test/unit"
#require "mathextension/vector3dinternal.rb"
#require "crystal/atom2.rb"
#require "crysna/sitenamelabeledcell.rb"

# assert_equal( cor, data)
# assert_in_delta( cor, data, $tolerance )
# assert_raise( RuntimeError ){}

class SiteNameLabeledCell
  public :nearest_atom_index
  public_class_method :nearest_site
  public_class_method :unique_sitenames?
end

class TC_SiteNameLabeledCell < Test::Unit::TestCase
  $tolerance = 1.0 * (10 **(-10))

  def setup
    @sites00 = {
      "site00" => Mageo::Vector3DInternal[ 0.01, 0.01, 0.01 ],
      "site01" => Mageo::Vector3DInternal[ 0.11, 0.21, 0.31 ]
    }
    @atoms00 = [
      CrystalCell::Atom.new( 0, [ 0.00, 0.00, 0.00 ]),
      CrystalCell::Atom.new( 0, [ 0.10, 0.20, 0.30 ])
    ]
    @c00 = SiteNameLabeledCell.new(
      [ [2.0, 2.0, 2.0], [0.0, 2.0, 2.0], [0.0, 0.0, 2.0] ], @atoms00, @sites00
    )
    @c00.comment = "c00"

    @sites01 = {
      "site00" => Mageo::Vector3DInternal[ 0.01, 0.01, 0.01 ],
      "site01" => Mageo::Vector3DInternal[ 0.11, 0.21, 0.31 ]
    }
    @atoms01 = [
      CrystalCell::Atom.new( 0, [ 0.99, 0.99, 0.99 ]),
      CrystalCell::Atom.new( 0, [ 0.90, 0.20, 0.30 ])
    ]
    @c01 = SiteNameLabeledCell.new(
      [ [2.0, 2.0, 2.0], [0.0, 2.0, 2.0], [0.0, 0.0, 2.0] ], @atoms01, @sites01
    )
  end

  def test_initialize
    # サイト数と原子数が一致しないときに例外。
    assert_raise( SiteNameLabeledCell::InitializeError ){
      SiteNameLabeledCell.new(
        [ [2.0, 2.0, 2.0], [0.0, 2.0, 2.0], [0.0, 0.0, 2.0] ],
        @atoms00,
        {}
      )
    }

    # 同上
    sites = Hash.new
    sites["site00"] = Mageo::Vector3DInternal[ 0.01, 0.01, 0.01 ]
    sites["site01"] = Mageo::Vector3DInternal[ 0.20, 0.20, 0.20 ]
    sites["site02"] = Mageo::Vector3DInternal[ 0.80, 0.80, 0.80 ]
    assert_raise( SiteNameLabeledCell::InitializeError ){
      SiteNameLabeledCell.new(
        [ [2.0, 2.0, 2.0], [0.0, 2.0, 2.0], [0.0, 0.0, 2.0] ],
        @atoms00,
        sites
      )
    }

    # 最近接の原子とサイト名が一対一対応しないときに例外。
    sites = Hash.new
    sites["site00"] = Mageo::Vector3DInternal[ 0.01, 0.01, 0.01 ]
    sites["site01"] = Mageo::Vector3DInternal[ 0.01, 0.01, 0.01 ]
    assert_raise( SiteNameLabeledCell::InitializeError ){
      SiteNameLabeledCell.new(
        [ [2.0, 2.0, 2.0], [0.0, 2.0, 2.0], [0.0, 0.0, 2.0] ],
        @atoms00,
        sites
      )
    }

    # 同上、ただし周期境界を考慮していることを確認。
    sites = Hash.new
    sites["site00"] = Mageo::Vector3DInternal[ 0.0, 0.0, 0.8 ]
    sites["site01"] = Mageo::Vector3DInternal[ 0.0, 0.0, 0.1 ]
    atoms = []
    atoms << CrystalCell::Atom.new( 0, [ 0.00, 0.00, 0.05 ])
    atoms << CrystalCell::Atom.new( 0, [ 0.00, 0.00, 0.99 ])
    assert_raise( SiteNameLabeledCell::InitializeError ){
      SiteNameLabeledCell.new(
        [ [2.0, 2.0, 2.0], [0.0, 2.0, 2.0], [0.0, 0.0, 2.0] ],
        atoms,
        sites
      )
    }

    axes = [ [2.0, 0.0, 0.0], [0.0, 2.0, 0.0], [0.0, 0.0, 2.0] ]
    sites = {
      "site00" => Mageo::Vector3DInternal[ 0.99, 0.99, 0.01 ],
      "site01" => Mageo::Vector3DInternal[ 0.90, 0.99, 0.01 ]
    }
    atoms = [
      CrystalCell::Atom.new( 0, [ 0.90, 0.01, 0.01 ]),
      CrystalCell::Atom.new( 0, [ 0.99, 0.01, 0.01 ])
    ]
    #
    assert_nothing_raised{ SiteNameLabeledCell.new(axes, atoms, sites) }
  end

  def test_nearest_atom_index
    assert_raise(SiteNameLabeledCell::TypeError){
      @c00.nearest_atom_index( Mageo::Vector3DInternal[0.0, 0.0, 0.0] )
    }

    assert_equal(0 , @c00.nearest_atom_index("site00"))
    assert_equal(1 , @c00.nearest_atom_index("site01"))
  end

  def test_sum_square_distances
    # 破壊的でないことを確認。
    sites = {
      "site00" => Mageo::Vector3DInternal[ 0.01, 0.01, 0.01 ],
    }
    atoms = [
      CrystalCell::Atom.new( 0, [-0.01,-0.01,-0.01 ], "site00"),
    ]
    cell = SiteNameLabeledCell.new(
      [ [1.0, 0.0, 0.0], [0.0, 1.0, 0.0], [0.0, 0.0, 1.0] ], atoms, sites
    )
    #pp cell
    assert_in_delta( 0.0012, cell.sum_square_distances, $tolerance)
    assert_in_delta(-0.01, cell.atoms[0].position[0], $tolerance)
    assert_in_delta(-0.01, cell.atoms[0].position[1], $tolerance)
    assert_in_delta(-0.01, cell.atoms[0].position[2], $tolerance)

    # 機能テスト
    assert_in_delta(0.0112 , @c00.sum_square_distances, $tolerance)

  end

  def test_to_cell
    t = @c00.to_cell
    assert_equal(CrystalCell::Cell, t.class)
    assert_equal(2, t.atoms.size)
    assert_equal(0, t.atoms[0].element)
    assert_equal("site00", t.atoms[0].name)
    assert_equal(Mageo::Vector3DInternal[0.0, 0.0, 0.0], t.atoms[0].position)
    assert_equal(0, t.atoms[1].element)
    assert_equal("site01", t.atoms[1].name)
    assert_equal(Mageo::Vector3DInternal[0.1, 0.2, 0.3], t.atoms[1].position)
    assert_equal("c00", t.comment)
  end

  def test_center_of_sites
    vec = @c00.center_of_sites
    assert_equal(Mageo::Vector3DInternal, vec.class)
    assert_in_delta(0.06, vec[0], $tolerance)
    assert_in_delta(0.11, vec[1], $tolerance)
    assert_in_delta(0.16, vec[2], $tolerance)
  end

  def test_center_of_atoms
    vec = @c00.center_of_atoms
    assert_equal(Mageo::Vector3DInternal, vec.class)
    assert_in_delta(0.05, vec[0], $tolerance)
    assert_in_delta(0.10, vec[1], $tolerance)
    assert_in_delta(0.15, vec[2], $tolerance)
  end

  def test_sitenames_positions
    t = @c00.sitenames_positions
    assert_equal(2, t.size)
    assert_equal(Mageo::Vector3DInternal, t["site00"].class)
    assert_in_delta(0.00, t["site00"][0], $tolerance)
    assert_in_delta(0.00, t["site00"][1], $tolerance)
    assert_in_delta(0.00, t["site00"][2], $tolerance)
    #
    assert_equal(Mageo::Vector3DInternal, t["site01"].class)
    assert_in_delta(0.10, t["site01"][0], $tolerance)
    assert_in_delta(0.20, t["site01"][1], $tolerance)
    assert_in_delta(0.30, t["site01"][2], $tolerance)

    t = @c01.sitenames_positions
    assert_equal(2, t.size)
    assert_equal(Mageo::Vector3DInternal, t["site00"].class)
    assert_in_delta(-0.01, t["site00"][0], $tolerance)
    assert_in_delta(-0.01, t["site00"][1], $tolerance)
    assert_in_delta(-0.01, t["site00"][2], $tolerance)
    #
    assert_equal(Mageo::Vector3DInternal, t["site01"].class)
    assert_in_delta(-0.10, t["site01"][0], $tolerance)
    assert_in_delta( 0.20, t["site01"][1], $tolerance)
    assert_in_delta( 0.30, t["site01"][2], $tolerance)
  end

  def test_sitenames_elements
    assert_equal(
      { "site00" => 0,
        "site01" => 0
      },
      @c00.sitenames_elements
    )
    assert_equal(
      { "site00" => 0,
        "site01" => 0
      },
      @c01.sitenames_elements
    )
  end
end

class TC_SiteNameLabeledCellClassMethods < Test::Unit::TestCase
  def setup
    @sites00 = {
      "site00" => Mageo::Vector3DInternal[ 0.01, 0.01, 0.01 ],
      "site01" => Mageo::Vector3DInternal[ 0.11, 0.21, 0.31 ]
    }
    @atoms00 = [
      CrystalCell::Atom.new( 0, [ 0.00, 0.00, 0.00 ]),
      CrystalCell::Atom.new( 0, [ 0.10, 0.20, 0.30 ])
    ]

    @sites01 = {
      "site00" => Mageo::Vector3DInternal[ 0.01, 0.01, 0.01 ],
      "site01" => Mageo::Vector3DInternal[ 0.11, 0.21, 0.31 ]
    }
    @atoms01 = [
      CrystalCell::Atom.new( 0, [ 0.99, 0.99, 0.99 ]),
      CrystalCell::Atom.new( 0, [ 0.90, 0.20, 0.30 ])
    ]
  end

  def test_self_unique_sitenames?
    atoms01 = [
      CrystalCell::Atom.new( 0, [ 0.00, 0.00, 0.00 ], "site00"),
      CrystalCell::Atom.new( 0, [ 0.00, 0.00, 0.00 ], "site01")
    ]
    assert_equal(true , SiteNameLabeledCell.unique_sitenames?(atoms01))
    
    atoms01 = [
      CrystalCell::Atom.new( 0, [ 0.00, 0.00, 0.00 ], "site00"),
      CrystalCell::Atom.new( 0, [ 0.00, 0.00, 0.00 ], "site00")
    ]
    assert_equal(false, SiteNameLabeledCell.unique_sitenames?(atoms01))
    
    atoms01 = [
      CrystalCell::Atom.new( 0, [ 0.00, 0.00, 0.00 ], "site00"),
      CrystalCell::Atom.new( 0, [ 0.00, 0.00, 0.00 ], nil)
    ]
    assert_equal(false, SiteNameLabeledCell.unique_sitenames?(atoms01))
  end

  def test_label_sitenames
    axes = [ [2.0, 2.0, 2.0], [0.0, 2.0, 2.0], [0.0, 0.0, 2.0] ]
    sites = @sites00
    atoms = @atoms00
    t = SiteNameLabeledCell.label_sitenames(axes, atoms, sites)
    assert_equal(2, t.size)
    assert_equal("site00", t[0].name)
    assert_in_delta(0.00, t[0].position[0])
    assert_in_delta(0.00, t[0].position[1])
    assert_in_delta(0.00, t[0].position[2])
    #
    assert_equal("site01", t[1].name)
    assert_in_delta(0.10, t[1].position[0])
    assert_in_delta(0.20, t[1].position[1])
    assert_in_delta(0.30, t[1].position[2])

    axes = [ [2.0, 2.0, 2.0], [0.0, 2.0, 2.0], [0.0, 0.0, 2.0] ]
    sites = @sites01
    atoms = @atoms01
    t = SiteNameLabeledCell.label_sitenames(axes, atoms, sites)
    assert_equal(2, t.size)
    assert_equal("site00", t[0].name)
    assert_in_delta( 0.99, t[0].position[0])
    assert_in_delta( 0.99, t[0].position[1])
    assert_in_delta( 0.99, t[0].position[2])
    #
    assert_equal("site01", t[1].name)
    assert_in_delta( 0.90, t[1].position[0])
    assert_in_delta( 0.20, t[1].position[1])
    assert_in_delta( 0.30, t[1].position[2])

    axes = [ [2.0, 0.0, 0.0], [0.0, 2.0, 0.0], [0.0, 0.0, 2.0] ]
    sites = {
      "site00" => Mageo::Vector3DInternal[ 0.99, 0.99, 0.01 ],
      "site01" => Mageo::Vector3DInternal[ 0.90, 0.99, 0.01 ]
    }
    #
    atoms = [
      CrystalCell::Atom.new( 0, [ 0.90, 0.01, 0.01 ]),
      CrystalCell::Atom.new( 0, [ 0.99, 0.01, 0.01 ])
    ]
    #
    t = SiteNameLabeledCell.label_sitenames(axes, atoms, sites)
    assert_equal(2, t.size)
    assert_in_delta( 0.90, t[0].position[0])
    assert_in_delta( 0.01, t[0].position[1])
    assert_in_delta( 0.01, t[0].position[2])
    assert_equal("site01", t[0].name)
    #
    assert_in_delta( 0.99, t[1].position[0])
    assert_in_delta( 0.01, t[1].position[1])
    assert_in_delta( 0.01, t[1].position[2])
    assert_equal("site00", t[1].name)

    axes = [ [2.0, 0.0, 0.0], [0.0, 2.0, 0.0], [0.0, 0.0, 2.0] ]
    #
    atoms = [
      CrystalCell::Atom.new( 0, [ 0.01, 0.01, 0.01 ]),
      CrystalCell::Atom.new( 0, [-0.89,-0.79,-0.69 ])
    ]
    sites = {
      "site00" => Mageo::Vector3DInternal[ 0.01, 0.01, 0.01 ],
      "site01" => Mageo::Vector3DInternal[ 0.11, 0.21, 0.31 ]
    }
    t = SiteNameLabeledCell.label_sitenames(axes, atoms, sites)
    assert_equal(2, t.size)
    assert_in_delta( 0.01, t[0].position[0])
    assert_in_delta( 0.01, t[0].position[1])
    assert_in_delta( 0.01, t[0].position[2])
    assert_equal("site00", t[0].name)
    #
    assert_in_delta(-0.89, t[1].position[0])
    assert_in_delta(-0.79, t[1].position[1])
    assert_in_delta(-0.69, t[1].position[2])
    assert_equal("site01", t[1].name)
  end

  def test_periodically_translate_atoms_to_nearest_site
    axes = [ [2.0, 2.0, 2.0], [0.0, 2.0, 2.0], [0.0, 0.0, 2.0] ]
    sites = @sites00
    atoms = [
      CrystalCell::Atom.new( 0, [ 0.00, 0.00, 0.00 ], "site00"),
      CrystalCell::Atom.new( 0, [ 0.10, 0.20, 0.30 ], "site01")
    ]
    t = SiteNameLabeledCell.periodically_translate_atoms_to_nearest_site(
      axes, atoms, sites)
    assert_equal(2, t.size)
    assert_equal("site00", t[0].name)
    assert_in_delta(0.00, t[0].position[0])
    assert_in_delta(0.00, t[0].position[1])
    assert_in_delta(0.00, t[0].position[2])
    #
    assert_equal("site01", t[1].name)
    assert_in_delta(0.10, t[1].position[0])
    assert_in_delta(0.20, t[1].position[1])
    assert_in_delta(0.30, t[1].position[2])

    axes = [ [2.0, 2.0, 2.0], [0.0, 2.0, 2.0], [0.0, 0.0, 2.0] ]
    sites = @sites01
    atoms = [
      CrystalCell::Atom.new( 0, [ 0.99, 0.99, 0.99 ], "site00"),
      CrystalCell::Atom.new( 0, [ 0.90, 0.20, 0.30 ], "site01")
    ]
    t = SiteNameLabeledCell.periodically_translate_atoms_to_nearest_site(
      axes, atoms, sites)
    assert_equal(2, t.size)
    assert_equal("site00", t[0].name)
    assert_in_delta(-0.01, t[0].position[0])
    assert_in_delta(-0.01, t[0].position[1])
    assert_in_delta(-0.01, t[0].position[2])
    #
    assert_equal("site01", t[1].name)
    assert_in_delta(-0.10, t[1].position[0])
    assert_in_delta( 0.20, t[1].position[1])
    assert_in_delta( 0.30, t[1].position[2])

    axes = [ [2.0, 0.0, 0.0], [0.0, 2.0, 0.0], [0.0, 0.0, 2.0] ]
    sites = {
      "site00" => Mageo::Vector3DInternal[ 0.99, 0.99, 0.01 ],
      "site01" => Mageo::Vector3DInternal[ 0.90, 0.99, 0.01 ]
    }
    #
    atoms = [
      CrystalCell::Atom.new( 0, [ 0.90, 0.01, 0.01 ], "site01"),
      CrystalCell::Atom.new( 0, [ 0.99, 0.01, 0.01 ], "site00")
    ]
    #
    t = SiteNameLabeledCell.periodically_translate_atoms_to_nearest_site(
      axes, atoms, sites)
    assert_equal(2, t.size)
    assert_in_delta( 0.90, t[0].position[0])
    assert_in_delta( 1.01, t[0].position[1])
    assert_in_delta( 0.01, t[0].position[2])
    assert_equal("site01", t[0].name)
    #
    assert_in_delta( 0.99, t[1].position[0])
    assert_in_delta( 1.01, t[1].position[1])
    assert_in_delta( 0.01, t[1].position[2])
    assert_equal("site00", t[1].name)

    axes = [ [2.0, 0.0, 0.0], [0.0, 2.0, 0.0], [0.0, 0.0, 2.0] ]
    #
    atoms = [
      CrystalCell::Atom.new( 0, [ 0.01, 0.01, 0.01 ], "site00"),
      CrystalCell::Atom.new( 0, [-0.89,-0.79,-0.69 ], "site01")
    ]
    sites = {
      "site00" => Mageo::Vector3DInternal[ 0.01, 0.01, 0.01 ],
      "site01" => Mageo::Vector3DInternal[ 0.11, 0.21, 0.31 ]
    }
    #
    t = SiteNameLabeledCell.periodically_translate_atoms_to_nearest_site(
      axes, atoms, sites)
    assert_equal(2, t.size)
    assert_in_delta( 0.01, t[0].position[0])
    assert_in_delta( 0.01, t[0].position[1])
    assert_in_delta( 0.01, t[0].position[2])
    assert_equal("site00", t[0].name)
    #
    assert_in_delta( 0.11, t[1].position[0])
    assert_in_delta( 0.21, t[1].position[1])
    assert_in_delta( 0.31, t[1].position[2])
    assert_equal("site01", t[1].name)

  end

  def test_self_nearest_site
    la = CrystalCell::LatticeAxes.new([ [2.0, 2.0, 2.0], [0.0, 2.0, 2.0], [0.0, 0.0, 2.0] ])
    pos = Mageo::Vector3DInternal[0.0, 0.0, 0.0]
    t = SiteNameLabeledCell.nearest_site(la, pos, @sites00)
    assert_equal("site00" , t)

    pos = Mageo::Vector3DInternal[0.1, 0.2, 0.3]
    t = SiteNameLabeledCell.nearest_site(la, pos, @sites00)
    assert_equal("site01" , t)
  end

  def test_labeled_all_atoms?
    atoms = [
      CrystalCell::Atom.new( 0, [ 0.00, 0.00, 0.00 ], "site00"),
      CrystalCell::Atom.new( 0, [ 0.10, 0.20, 0.30 ], "site01")
    ]
    assert_equal(true, SiteNameLabeledCell.labeled_all_atoms?(atoms, @sites00))

    atoms = [
      CrystalCell::Atom.new( 0, [ 0.00, 0.00, 0.00 ], "site00"),
      CrystalCell::Atom.new( 0, [ 0.10, 0.20, 0.30 ], "site00")
    ]
    assert_equal(true, SiteNameLabeledCell.labeled_all_atoms?(atoms, @sites00))

    atoms = [
      CrystalCell::Atom.new( 0, [ 0.00, 0.00, 0.00 ], "site00"),
      CrystalCell::Atom.new( 0, [ 0.10, 0.20, 0.30 ])
    ]
    assert_equal(false, SiteNameLabeledCell.labeled_all_atoms?(atoms, @sites00))

    atoms = [
      CrystalCell::Atom.new( 0, [ 0.00, 0.00, 0.00 ]),
      CrystalCell::Atom.new( 0, [ 0.10, 0.20, 0.30 ], "site00")
    ]
    assert_equal(false, SiteNameLabeledCell.labeled_all_atoms?(atoms, @sites00))

    atoms = [
      CrystalCell::Atom.new( 0, [ 0.00, 0.00, 0.00 ]),
      CrystalCell::Atom.new( 0, [ 0.10, 0.20, 0.30 ])
    ]
    assert_equal(false, SiteNameLabeledCell.labeled_all_atoms?(atoms, @sites00))

  end

  def test_self_check_inside_all_sites
    assert_nothing_raised{SiteNameLabeledCell.check_inside_all_sites(@sites00)}
    assert_nothing_raised{SiteNameLabeledCell.check_inside_all_sites(@sites01)}

    sites = {
      "site00" => Mageo::Vector3DInternal[ 0.00, 0.00, 0.00 ],
      "site01" => Mageo::Vector3DInternal[ 0.11, 0.21, 0.31 ]
    }
    assert_nothing_raised{SiteNameLabeledCell.check_inside_all_sites(sites)}

    sites = {
      "site00" => Mageo::Vector3DInternal[ 1.01, 0.00, 0.00 ],
      "site01" => Mageo::Vector3DInternal[ 0.11, 0.21, 0.31 ]
    }
    assert_raise(SiteNameLabeledCell::OutOfCellError){
      SiteNameLabeledCell.check_inside_all_sites(sites)}

    sites = {
      "site00" => Mageo::Vector3DInternal[-0.01, 0.00, 0.00 ],
      "site01" => Mageo::Vector3DInternal[ 0.11, 0.21, 0.31 ]
    }
    assert_raise(SiteNameLabeledCell::OutOfCellError){
      SiteNameLabeledCell.check_inside_all_sites(sites)}
  end

  #def test_site_name
  # assert_equal("site00", @c00.site_name(Mageo::Vector3DInternal[ 0.01, 0.01, 0.01 ]))
  # assert_equal("site00", @c00.site_name(Mageo::Vector3DInternal[ 0.00, 0.00, 0.00 ]))
  # assert_equal("site01", @c00.site_name(Mageo::Vector3DInternal[ 0.11, 0.21, 0.31 ]))
  # assert_equal("site01", @c00.site_name(Mageo::Vector3DInternal[ 0.10, 0.20, 0.30 ]))

  # assert_raise(SiteNameLabeledCell::TypeError){
  #       @c00.site_name(Mageo::Vector3D[ 0.10, 0.20, 0.30 ]))
  # }
  #end

end

class TC_Crystalcell_Cell < Test::Unit::TestCase
  def setup
    # 元素の識別子を数字にしたもの。
    atoms = [
      CrystalCell::Atom.new( 0, [0.0, 0.0, 0.0] ),
      CrystalCell::Atom.new( 1, [0.1, 0.2, 0.3] ),
    ]
    @c00 = CrystalCell::Cell.new( [ [2.0, 2.0, 2.0], [0.0, 2.0, 2.0], [0.0, 0.0, 2.0] ], atoms )
    @c00.comment = "c01"
  end

  def test_to_snlcell
    sites = {
      "site00" => Mageo::Vector3DInternal[ 0.01, 0.01, 0.01 ],
      "site01" => Mageo::Vector3DInternal[ 0.11, 0.21, 0.31 ]
    }

    t = @c00.to_snlcell( sites )
    assert_equal(SiteNameLabeledCell, t.class)
    assert_equal( 2, t.atoms.size)
    assert_equal( "site00", t.atoms[0].name)
    assert_equal( "site01", t.atoms[1].name)
    assert_equal( "c01", t.comment)
  end
end
