#! /usr/bin/env ruby
# coding: utf-8

require "helper"
#require "test/unit"
#require "pkg/klass.rb"

class Crysna::Cell
  attr_accessor :atoms
  attr_accessor :reach_flag
  attr_accessor :from_edges
end

class TC_TFCell < Test::Unit::TestCase
  def setup
    @cell00 = Crysna::TransitionFinder::Cell.new(
      { 0 => ["A", "B"], 1 => ["E"]}, -10.0)

    atoms = [
      Crysna::Atom.new(0, "A", [1, 0, 0]),
      Crysna::Atom.new(0, "B", [1, 0, 0]),
      Crysna::Atom.new(1, "F", [1, 0, 0]),
    ]
    @cell01 = Crysna::TransitionFinder::Cell.new({}, -11.0)
    @cell01.atoms = atoms
    @cell02 = Crysna::TransitionFinder::Cell.new({}, -11.0, "abc")
    @cell02.atoms = atoms
  end

  def test_equal
    assert_equal(true , @cell00 == @cell00)
    assert_equal(true , @cell01 == @cell01)
    assert_equal(true , @cell02 == @cell02)
    assert_equal(false, @cell00 == @cell01)
    assert_equal(false, @cell01 == @cell02)
    assert_equal(false, @cell02 == @cell00)
  end

  def test_same_atoms?
    cell = Crysna::TransitionFinder::Cell.new({}, -10.0)
    cell.atoms = [
      Crysna::Atom.new(0, "A", [0, 0, 0]),
      Crysna::Atom.new(0, "B", [0, 0, 0]),
      Crysna::Atom.new(1, "E", [0, 0, 0]),
    ]
    assert_equal(true, @cell00.same_atoms?(cell))

    cell = Crysna::TransitionFinder::Cell.new({}, -11.0)
    cell.atoms = [
      Crysna::Atom.new(0, "A", [0, 0, 0]),
      Crysna::Atom.new(0, "B", [0, 0, 0]),
      Crysna::Atom.new(1, "E", [0, 0, 0]),
    ]
    assert_equal(true, @cell00.same_atoms?(cell))

    cell = Crysna::TransitionFinder::Cell.new({}, -10.0)
    cell.atoms = [
      Crysna::Atom.new(0, "A", [1, 0, 0]),
      Crysna::Atom.new(0, "B", [0, 0, 0]),
      Crysna::Atom.new(1, "E", [0, 0, 0]),
    ]
    assert_equal(false, @cell00.same_atoms?(cell))

    cell = Crysna::TransitionFinder::Cell.new({}, -10.0)
    cell.atoms = [
      Crysna::Atom.new(0, "A", [0, 0, 0]),
      Crysna::Atom.new(0, "C", [0, 0, 0]),
      Crysna::Atom.new(1, "E", [0, 0, 0]),
    ]
    assert_equal(false, @cell00.same_atoms?(cell))

    cell = Crysna::TransitionFinder::Cell.new({}, -10.0)
    cell.atoms = [
      Crysna::Atom.new(0, "A", [0, 0, 0]),
      Crysna::Atom.new(0, "B", [0, 0, 0]),
      Crysna::Atom.new(0, "E", [0, 0, 0]),
    ]
    assert_equal(false, @cell00.same_atoms?(cell))

    cell = Crysna::TransitionFinder::Cell.new({}, -10.0)
    cell.atoms = [
      Crysna::Atom.new(0, "A", [0, 0, 0]),
      Crysna::Atom.new(0, "B", [0, 0, 0]),
      Crysna::Atom.new(1, "E", [0, 0, 0]),
    ]
    cell.reach_flag = true
    assert_equal(true, @cell00.same_atoms?(cell))

    cell = Crysna::TransitionFinder::Cell.new({}, -10.0)
    cell.atoms = [
      Crysna::Atom.new(0, "A", [0, 0, 0]),
      Crysna::Atom.new(0, "B", [0, 0, 0]),
      Crysna::Atom.new(1, "E", [0, 0, 0]),
    ]
    cell.from_edges = [nil]
    assert_equal(true, @cell00.same_atoms?(cell))

    cell = Crysna::TransitionFinder::Cell.new({}, -10.0)
    cell.atoms = [
      Crysna::Atom.new(0, "A", [0, 0, 0]),
      Crysna::Atom.new(0, "B", [0, 0, 0]),
      Crysna::Atom.new(1, "E", [0, 0, 0]),
    ]
    cell.name = "something"
    assert_equal(true, @cell00.same_atoms?(cell))
  end

  def test_same_sites?
    assert_equal(true , @cell00.same_sites?(@cell00))
    assert_equal(true , @cell01.same_sites?(@cell01))
    assert_equal(true , @cell02.same_sites?(@cell02))
    assert_equal(false, @cell00.same_sites?(@cell01))
    assert_equal(true , @cell01.same_sites?(@cell02))
    assert_equal(false, @cell02.same_sites?(@cell00))

    atoms = [
      Crysna::Atom.new(0, "A", [1, 0, 0]),
      Crysna::Atom.new(0, "B", [0, 0, 0]),
      Crysna::Atom.new(1, "F", [1, 0, 0]),
    ]
    cell = Crysna::TransitionFinder::Cell.new({}, -11.0)
    cell.atoms = atoms
    assert_equal(true , @cell01.same_sites?(cell))
  end

  def test_name
    assert_equal("", @cell00.name)
    assert_equal("", @cell01.name)
    assert_equal("abc", @cell02.name)

    cell = Crysna::TransitionFinder::Cell.new(
      { 0 => ["A", "B"], 1 => ["E"]}, -10.0, "abc")
    assert_equal("abc", cell.name)
  end

  def test_migrate!
    @cell01.migrate!(0, "C", [-1, 1, -1])
    atoms = @cell01.atoms
    assert_equal(Crysna::Atom.new(0, "B", [1,0, 0]), atoms.shift)
    assert_equal(Crysna::Atom.new(0, "C", [0,1,-1]), atoms.shift)
    assert_equal(Crysna::Atom.new(1, "F", [1,0, 0]), atoms.shift)
    assert_equal(nil                                                                     , atoms.shift)

    assert_raise(Crysna::TransitionFinder::Cell::RangeError){
      @cell01.migrate!(3, "C", [-1, 1, -1])
    }
  end

  def test_migrate
    result = @cell01.migrate(0, "C", [-1, 1, -1])
    atoms = @cell01.atoms
    assert_equal(Crysna::Atom.new(0, "A", [1,0, 0]), atoms.shift)
    assert_equal(Crysna::Atom.new(0, "B", [1,0, 0]), atoms.shift)
    assert_equal(Crysna::Atom.new(1, "F", [1,0, 0]), atoms.shift)
    assert_equal(nil                                                                     , atoms.shift)

    atoms = result.atoms
    assert_equal(Crysna::Atom.new(0, "B", [1,0, 0]), atoms.shift)
    assert_equal(Crysna::Atom.new(0, "C", [0,1,-1]), atoms.shift)
    assert_equal(Crysna::Atom.new(1, "F", [1,0, 0]), atoms.shift)
    assert_equal(nil                                                                 , atoms.shift)

  end

  def test_reach
    assert_equal(false, @cell00.reach?)
    assert_equal([]  , @cell00.from_edges)
    @cell00.reach(nil)
    assert_equal(true, @cell00.reach?)
    assert_equal([nil], @cell00.from_edges)

    setup
    edge = Crysna::TransitionFinder::Edge.new([0,1], 0.0, "")
    @cell00.reach(edge)
    assert_equal(true, @cell00.reach?)
    assert_equal([edge], @cell00.from_edges)
  end

  def test_periodically_equal?
    assert_equal(false, @cell00.periodically_equal?(@cell01))
    assert_equal(false, @cell01.periodically_equal?(@cell00))
    assert_equal(true , @cell00.periodically_equal?(@cell00))
    assert_equal(true , @cell01.periodically_equal?(@cell01))

    atoms = [
      Crysna::Atom.new(0, "A", [1, 0, 0]),
      Crysna::Atom.new(0, "B", [1, 1, 0]),
      Crysna::Atom.new(1, "F", [1, 0, 0]),
    ]
    tfc02 = Crysna::TransitionFinder::Cell.new({}, -11.0)
    tfc02.atoms = atoms
    assert_equal(true , @cell01.periodically_equal?(tfc02))

  end

  #def test_reset
  #    edge = Crysna::TransitionFinder::Edge.new([0,1], 0.0, "")
  #    @cell00.from_edges = [edge]
  #    @cell00.reach_flag = true
  #    assert_equal(true , @cell00.reach_flag)
  #    assert_equal([edge], @cell00.from_edges)
  #    @cell00.reset
  #    assert_equal(false , @cell00.reach_flag)
  #    assert_equal([], @cell00.from_edges)
  #end

  def test_occupied_sitename
    assert_equal('A555-B555-E555', @cell00.occupied_sitename)
    assert_equal('A655-B655-F655', @cell01.occupied_sitename)
    assert_equal('A655-B655-F655', @cell02.occupied_sitename)
  end

  def test_inspect_short
    assert_equal('Crysna::TransitionFinder::Cell: A555-B555-E555, @energy = -10.0, @reach_flag = false, @from_edges = [], @name = ""', @cell00.inspect_short)
    assert_equal('Crysna::TransitionFinder::Cell: A655-B655-F655, @energy = -11.0, @reach_flag = false, @from_edges = [], @name = ""', @cell01.inspect_short)
    assert_equal('Crysna::TransitionFinder::Cell: A655-B655-F655, @energy = -11.0, @reach_flag = false, @from_edges = [], @name = "abc"', @cell02.inspect_short)
  end

  def test_operate
    rotation0 = [
      [1, 0, 0],
      [0, 1, 0],
      [0, 0, 1],
    ]
    rotation1 = [
      [-1, 0, 0],
      [0, 1, 0],
      [0, 0, 1],
    ]
    operations = [
      {
        "A" => ["A", [ 0, 0, 0], rotation0],
        "B" => ["B", [ 0, 0, 0], rotation0],
        "C" => ["C", [ 0, 0, 0], rotation0],
        "D" => ["D", [ 0, 0, 0], rotation0],
        "E" => ["E", [ 0, 0, 0], rotation0],
        "F" => ["F", [ 0, 0, 0], rotation0],
        "G" => ["G", [ 0, 0, 0], rotation0],
      },
      {
        "A" => ["D", [ 0, 0, 0], rotation1],
        "B" => ["C", [ 0, 0, 0], rotation1],
        "C" => ["B", [ 0, 0, 0], rotation1],
        "D" => ["A", [ 0, 0, 0], rotation1],
        "E" => ["G", [ 0, 0, 0], rotation1],
        "F" => ["F", [ 0, 0, 0], rotation1],
        "G" => ["E", [ 0, 0, 0], rotation1],
      }
    ]
    cell0 = Crysna::TransitionFinder::Cell.new({}, -10.0)
    cell0.atoms = [
      Crysna::Atom.new(0, "A", [0, 0, 0]),
      Crysna::Atom.new(0, "B", [0, 0, 0]),
      Crysna::Atom.new(1, "E", [0, 0, 0]),
    ]
    cell1 = Crysna::TransitionFinder::Cell.new({}, -10.0)
    cell1.atoms = [
      Crysna::Atom.new(0, "C", [0, 0, 0]),
      Crysna::Atom.new(0, "D", [0, 0, 0]),
      Crysna::Atom.new(1, "G", [0, 0, 0]),
    ]
    assert_equal(cell0, cell0.operate(operations[0]))
    assert_equal(cell1, cell0.operate(operations[1]))
    assert_equal(cell1, cell1.operate(operations[0]))
    assert_equal(cell0, cell1.operate(operations[1]))


    cell0 = Crysna::TransitionFinder::Cell.new({}, -10.0)
    cell0.atoms = [
      Crysna::Atom.new(0, "A", [1, 0, 0]),
      Crysna::Atom.new(0, "B", [1, 0, 0]),
      Crysna::Atom.new(1, "E", [1, 0, 0]),
    ]
    cell1 = Crysna::TransitionFinder::Cell.new({}, -10.0)
    cell1.atoms = [
      Crysna::Atom.new(0, "C", [-1, 0, 0]),
      Crysna::Atom.new(0, "D", [-1, 0, 0]),
      Crysna::Atom.new(1, "G", [-1, 0, 0]),
    ]
    assert_equal(cell0, cell0.operate(operations[0]))
    assert_equal(cell1, cell0.operate(operations[1]))
    assert_equal(cell1, cell1.operate(operations[0]))
    assert_equal(cell0, cell1.operate(operations[1]))
  end

end

