#! /usr/bin/env ruby
# coding: utf-8

require "helper"
#require "test/unit"
#require "pkg/klass.rb"

class Crysna::TransitionFinder::Cell
  attr_accessor :atoms
  attr_accessor :reach_flag
end

class Crysna::TransitionFinder::CellManager
  attr_accessor :cells
end

class TC_CellManager < Test::Unit::TestCase

  def setup
    ##Cell
    @cell_ABE = Crysna::TransitionFinder::Cell.new({0 => ["A", "B"], 1 => ["E"]}, -10.0)
    @cell_ABEr = Marshal.load(Marshal.dump(@cell_ABE))
    @cell_ABEr.reach_flag = true

    @cell_ACE = Crysna::TransitionFinder::Cell.new({0 => ["A", "C"], 1 => ["E"]}, -10.0)
    @cell_ACEr = Marshal.load(Marshal.dump(@cell_ACE))
    @cell_ACEr.reach_flag = true

    @cell_ABEp = Crysna::TransitionFinder::Cell.new({}, -12.0)
    @cell_ABEp.atoms = [
      Crysna::Atom.new(0, "A", [0, 0, 0]),
      Crysna::Atom.new(0, "B", [1, 0, 0]),
      Crysna::Atom.new(1, "E", [0, 0, 0]),
    ]
    @cell_ABEpr = Marshal.load(Marshal.dump(@cell_ABEp))
    @cell_ABEpr.reach_flag = true

    @cell_ACpE = Crysna::TransitionFinder::Cell.new({}, -12.0)
    @cell_ACpE.atoms = [
      Crysna::Atom.new(0, "A", [0, 0, 0]),
      Crysna::Atom.new(0, "C", [1, 0, 0]),
      Crysna::Atom.new(1, "E", [0, 0, 0]),
    ]
    @cell_ACpEr = Marshal.load(Marshal.dump(@cell_ACpE))
    @cell_ACpEr.reach_flag = true

    @cell_ADpE = Crysna::TransitionFinder::Cell.new({}, -12.0)
    @cell_ADpE.atoms = [
      Crysna::Atom.new(0, "A", [0, 0, 0]),
      Crysna::Atom.new(0, "D", [1, 0, 0]),
      Crysna::Atom.new(1, "E", [0, 0, 0]),
    ]
    @cell_ADpEr = Marshal.load(Marshal.dump(@cell_ADpE))
    @cell_ADpEr.reach_flag = true

    @cell_ACmE = Crysna::TransitionFinder::Cell.new({}, -12.0)
    @cell_ACmE.atoms = [
      Crysna::Atom.new(0, "A", [ 0, 0, 0]),
      Crysna::Atom.new(0, "C", [-1, 0, 0]),
      Crysna::Atom.new(1, "E", [ 0, 0, 0]),
    ]
    @cell_ACmEr = Marshal.load(Marshal.dump(@cell_ACmE))
    @cell_ACmEr.reach_flag = true

    ##CellManager
    @cm00 = Crysna::TransitionFinder::CellManager.new

    @cm01 = Crysna::TransitionFinder::CellManager.new
    @cm01.cells = [
      @cell_ABE,
      @cell_ACE,
    ]

    @cm02 = Crysna::TransitionFinder::CellManager.new
    @cm02.cells = [
      @cell_ABE,
      @cell_ACE,
      @cell_ABEp,
      @cell_ACpE,
      @cell_ADpE,
      @cell_ACmE,
    ]

    @cm03 = Crysna::TransitionFinder::CellManager.new
    @cm03.cells = [
      @cell_ABEr,
      @cell_ACEr,
      @cell_ABEpr,
      @cell_ACpEr,
      @cell_ADpEr,
      @cell_ACmE,
    ]
  end

  def test_add
    result = @cm00.add(@cell_ABE)
    assert_equal(0, result)
    assert_equal(@cell_ABE, @cm00.cells[0])
    assert_equal(1, @cm00.cells.size)

    #already added
    result = @cm00.add(@cell_ABE)
    assert_equal(0, result)
    assert_equal(@cell_ABE, @cm00.cells[0])
    assert_equal(1, @cm00.cells.size)

    #new
    result = @cm00.add(@cell_ACE)
    assert_equal(1, result)
    assert_equal(@cell_ABE, @cm00.cells[0])
    assert_equal(@cell_ACE, @cm00.cells[1])
    assert_equal(2, @cm00.cells.size)
  end

  def test_fetch
    assert_equal(nil, @cm00[0])
    assert_equal(@cell_ABE, @cm01[0])
  end

  def test_index_same_atoms
    assert_equal(0, @cm01.index_same_atoms(@cell_ABE))
    assert_equal(1, @cm01.index_same_atoms(@cell_ACE))
    assert_equal(nil, @cm01.index_same_atoms(@cell_ABEp))
  end

  def test_index_same_sites
    assert_equal(0, @cm01.index_same_sites(@cell_ABE))
    assert_equal(1, @cm01.index_same_sites(@cell_ACE))

    #pp @cm01
    #pp @cell_ABEp
    assert_equal(0, @cm01.index_same_sites(@cell_ABEp))
  end

  def test_reach
    #pp @cm01
    assert_equal(false, @cm01.cells[0].reach?)
    assert_equal(false, @cm01.cells[1].reach?)
    assert_equal([],     @cm01.cells[1].from_edges)
    edge = Crysna::TransitionFinder::Edge.new([0,1], 0.0, "")
    @cm01.reach(1, edge)
    assert_equal(false, @cm01.cells[0].reach?)
    assert_equal(true , @cm01.cells[1].reach?)
    assert_equal([edge],     @cm01.cells[1].from_edges)
  end

  def test_periodically_shift_indices
    assert_equal([], @cm00.periodically_shift_indices)

    assert_equal([], @cm01.periodically_shift_indices)

    results = @cm02.periodically_shift_indices
    assert_equal( [], results
    )

    results = @cm03.periodically_shift_indices
    assert_equal( [ [0, 2], [1, 3], [4]], results
    )
  end

  def test_contain_periodically_shift_cell?
    assert_equal(false, @cm00.contain_periodically_shift_cells?)
    assert_equal(false, @cm01.contain_periodically_shift_cells?)
    assert_equal(false, @cm02.contain_periodically_shift_cells?)
    assert_equal(true , @cm03.contain_periodically_shift_cells?)
  end

  def test_ascend
    @cm02.reach(0, nil)
    edge = Crysna::TransitionFinder::Edge.new([0,2], 0.0, "")
    @cm02.reach(2, edge)
    edge = Crysna::TransitionFinder::Edge.new([2,1], 0.0, "")
    @cm02.reach(1, edge)
    assert_equal([1,2,0], @cm02.ascend(1))
  end

  #def test_to_s
  #    #assert_equal("Crysna::TransitionFinder::CellManager: [ABE000000000, @energy = -10.0, @reach_flag = false, @previous_cell = ACE000000000, @energy = -10.0, @reach_flag = false, @previous_cell = ]", @cm01.to_s)
  #    #"ABE000000000
  #    #@cm01.add(@cell_ABE)
  #    #@cm01.add(@cell_ACE)
  #end

end

